* CODE FOR BLOG ON ALTERNATIVE DISCOUNT RATES FOR ODA LOANS, AND WHAT IT DOES TO THEIR VALUE

use "C:\Users\eritchie\Downloads\crs_dac_1995-2018.dta", clear  // This is the collated bulk download CRS, with only DAC countries selected https://stats.oecd.org/DownloadFiles.aspx?DatasetCode=CRS1
keep if flowcode==11 | flowcode==13 | flowcode==19 //these are ODA flowcodes (others are 14 (OOF loans) and 30 (private loans)
keep if year>2002	// data before 2006 is quite bad, but want to make sure main relief years included. Including just years from 2006 increases calculated risk but only slightly
compress

gen group = incomegroupn
replace group = "LDCs" if group=="Other LICs" //they have same discount rate so no difference
replace group = "UMICs" if group=="MADCTs"	  //MADCTs are rich countries about to graduate, so giving highest interest rate

gen forgave = usd_disbursement if finance_t==610	// 610 is forgiven ODA principal (code for ODA principal rescheduled, 620, has no observations)
merge m:1 year recipientname using "C:\Users\eritchie\OneDrive - Center for Global Development\FID 2020\wb_historic_incomestatus"	// from world bank OGHIST 

/*The following countries/territories did not match, and so don't have historic income status. 
They'll just be given their 2018 value. But only two (Wallis and Futuna, and Cook Islands) have outstanding debt, and it's very small:
table recipientn _merge if _merge<3 & regex(group,"MICs"), c(sum usd_outstanding) f(%13.0fc)
	
Anguilla	0
Cook Islands	12

Montserrat	0	
Niue	0
Saint Helena	0
Tokelau	0
Wallis and Futuna	143
No LDCs are missing. 
*/	

drop if year == 2019
drop if _merge==2
replace inc_status = "UMICs" if inc_status=="UM"
replace inc_status = "UMICs" if inc_status=="H" // Countries with high income in 2018 wouldn't have had successful merges, this is in case any countries fell back 
												// (keeping is generous and really should probably remove, but makes little difference)
replace inc_status = "LMICs" if inc_status=="LM"
replace inc_status = "LDCs" if inc_status=="L"
replace inc_status = group if inc_status==".."

/* merge on historic ODA classifications */
drop _merge
merge m:1 recipientname using "C:\Users\eritchie\OneDrive - Center for Global Development\FID 2020\oda_historic_incstatus" // collated from http://www.oecd.org/development/financing-sustainable-development/development-finance-standards/historyofdaclistsofaidrecipientcountries.htm

replace gr1996 = inc_status if gr1996==""


*table year inc_status, c(sum forgave) f(%13.0fc)
*table year inc_status, c(sum usd_ou) f(%13.0fc)

bysort year inc_status: egen debtsum = sum(usd_outstanding)
bysort year inc_status: egen debtforgiven = sum(forgave)
bysort year inc_status: gen n = _n == 1
gen ratio = debtforgiven/debtsum if n==1

* Sort of assuming risk free rate is 2. This calculates the average annual debt forgiven as % of debt outstanding, by group
su ratio if inc_status=="LDCs"
local ldc = r(mean) + 0.02
su ratio if inc_status=="LMICs"
local lmic = r(mean) + 0.02
su ratio if inc_status=="UMICs"
local umic = r(mean) + 0.02

di `ldc' - 0.02
di `lmic' - 0.02
di `umic' - 0.02

/*____________________

This code works out the percentage default each year by income group, but using groups as of 1996.
This is because it is impossible to measure according to what groups countries were in at time loans were made (this doesn't seem to be what commitmentdate
measures) and so it is just to see if it materially changes results. 
Turns it out does, but makes the mismatch between implied risk and actual risk even more dramatic, and lowers ODA even more. 
So, using income group at time of default is actually generous to DAC. 
This is all referred to in passing in the blog, but not part of the published charts/figures

bysort year gr1996: egen debtsum2 = sum(usd_outstanding)
bysort year gr1996: egen debtforgiven2 = sum(forgave)
bysort year gr1996: gen n2 = _n == 1
gen ratio2 = debtforgiven2/debtsum2 if n==1

* Sort of assuming risk free rate is 2. This calculates the average annual debt forgiven as % of debt outstanding, by group
su ratio2 if inc_status=="LDCs"
local ldc2 = r(mean) + 0.02
su ratio2 if inc_status=="LMICs"
local lmic2 = r(mean) + 0.02
su ratio2 if inc_status=="UMICs"
local umic2 = r(mean) + 0.02

di `ldc2'
di `lmic2'
di `umic2'

*_____________________________
*/

qui {

keep if year>2014
*###############################################################################

*####  	This is working out grant equivalent with new discount rates

*###############################################################################

*COMMITMENT YEAR
	gen crsid2 = substr(crsid,1,10)
	gen commityear = substr(commitmentdate,1,4)
	destring commityear, replace
	gen crsyear = substr(crsid,1,4)
	destring crsyear, replace force
	replace commityear = crsyear if commityear==.
	replace commityear = crsyear if commityear==1900

	*some commitment years from crsid codes are above year, which must be input error (usually difference of 1, so probably year of entry into data?)
	replace commityear = year if commityear >year & commityear!=.

	*some commitment years don't make sense, in particular lots clearly listed as data input year. 
	*Initial report ==3 suggests not new commitment, so shouldn't have different commitmentdate to previous. Filling with mode usually works (see below)
	bysort donorname crsid2: egen modeyr = mode(commityear), maxmode
	replace commityear = modeyr if initialreport==3 & commityear>modeyr

replace typer = 1 if typer==.	// vast majority are type repayment (EPP). Type is changed below when repay1 and repay2 dates are same (which implies lumpsum)

*INTEREST RATES
	replace interest1 = subinstr(interest1,"'","",.)
	gen int1 = interest1
	destring int1, force replace
	gen I = int1/100000
	replace I = interest2/100000 if I==.
	replace I = interest2/100000 if I==0 & interest2!=. // end results revealed that many zero entries for interest1 had >0 entries for interest2 in 2008, and this skewed GE
	*There seems to be an issue with Stata not recognising two numbers as same when they were in a different string format. 
	*Should ask about this on statalist, but for now, rounding to the highest np.of decimal points in data (0.0001% - don't care about that level of precision anyway)
	replace I = round(I,0.00001)		
	bysort donorname: egen Imean = mean(I)
	replace I = Imean if I==. & flowcode==13		// there are a few missing values for I, so giving them mean by donor

*######DISCOUNT RATES
	*New (data driven) discount rates
		gen group2 = group
		*Some loans are to "Part I: unallocated by income". To be conservative we give strictest discount rate to these loans
		replace group2 = "UMICs" if strpos(group,"Part I")>0
		gen Dnew = 0
		replace Dnew = `ldc'  if group2=="LDCs"
		replace Dnew = `lmic' if group2=="LMICs"
		replace Dnew = `umic' if group2=="UMICs"

	*DAC discount rates
		gen D = 0
		replace D = 0.09 if group2=="LDCs"
		replace D = 0.07 if group2=="LMICs"
		replace D = 0.06 if group2=="UMICs"

*NUMBER REPAYMENTS
	rename numberrepayment N
	bysort donorname: egen Nmode = mode(N), maxmode
	replace N = Nmode if N==. & typerep!=3 //filling in any missing gaps for N (aren't many - not a big problem)
	replace N = Nmode if N==0

*################################# REPAY DATES
		gen repay1 = substr(repaydate1,1,4)		//year of first repayment
		gen repay2 = substr(repaydate2,1,4)		//year of final repayment
		destring repay1, replace
		destring repay2, replace

		*get rid of nonsensical values 	
		replace repay2 = . if repaydate1==repaydate2 & repaydate2!="" & typere!=3
		replace repay1=. if repay1<commityear	

		gen duration = repay2-commityear
		gen grace = repay1-commityear
		*Some observations have exactly same dates for repayment1 and repayment2, and also typerepayment==1. This implies only one repayment, so we change type to "lump sum".
		*(Then again, number of repayments =2 for some, so generally inconsistent. We assume repayment dates are correct, for want of more information). 
		replace typere = 3 if repay1==repay2 & repay1!=.
		
	bysort donorname: egen grace_av    = mean(grace)	//Some have repayment dates missing so just imputing the average. Not issue for many observations post2015
	bysort donorname: egen duration_av = mean(duration)
	replace grace_av    = round(grace_av)
	replace duration_av = round(duration_av)
	replace grace    = grace_av    if grace==.
	replace duration = duration_av if duration==.
	gen p = duration - grace



*#################################################

*Calculate grant element with DAC discount rates

*----- This is with the old discount rates to ensure it is reasonably close to DAC figures

		gen dd = (1+D)^(1/N) - 1

		*EQUAL PRINCIPAL PAYMENTS
		gen epp = (1 - (I/N) / dd) * ( 1 - (1/(1+dd)^(grace*N) - 1/(1+dd)^(duration*N)) / (dd*p*N))

		*LUMP SUM 
		gen lumpsum = 1 - (1+I*duration)/((1+D)^duration)

		*ANNUITY 
		gen dg = (1+D)^grace
		gen dp = (1+D)^p
		gen PVg = I * ( (1-1/dg)/(N*dd))
		
		gen PVn = (I/N) * ((1/((1+I/N)^(p*N) - 1)) + 1) * (1/dg) * ((1-1/dp)/dd)
		gen annuity = 1-PVg-PVn
		drop PV* dg dp

		*GRANT ELEMENT
		gen ge_calc = 0 // This is "grant element calculated", ie. the grant element worked out using the info provided
		replace ge_calc = epp if typerepayment==1
		replace ge_calc = annuity if typerepayment==2
		replace ge_calc = lumpsum if typerepayment==3
		replace ge_calc = epp if typerepayment==5 //This is an estimate, as need some value for "other" category. Generally not much difference between annuity and EPP, and repayment dates suggest not lump sum
		replace ge_calc = lumpsum if typere==5 & N==. // Investigation showed N only == . when type is lumpsum. Applying similar logic to estimating "other" category (little else can be done)
		
		replace ge_calc = 0 if ge_calc<0


gen impliedge = usd_grantequiv/usd_disbursement
gen geq = ge_calc*usd_disbursement


*#################################################

*----- Grant element with the NEW DISCOUNT RATES


gen ddnew = (1+Dnew)^(1/N) - 1
		
		*EQUAL PRINCIPAL PAYMENTS
		gen eppnew = (1 - (I/N) / ddnew) * ( 1 - (1/(1+ddnew)^(grace*N) - 1/(1+ddnew)^(duration*N)) / (ddnew*p*N))

		*LUMP SUM 
		gen lumpsumnew = 1 - (1+I*duration)/((1+Dnew)^duration)

		*ANNUITY 
		gen dg = (1+Dnew)^grace
		gen dp = (1+Dnew)^p
		gen PVg = I * ( (1-1/dg)/(N*ddnew))
		gen PVn = (I/N) * ((1/((1+I/N)^(p*N) - 1)) + 1) * (1/dg) * ((1-1/dp)/ddnew)
		gen annuitynew = 1-PVg-PVn
		drop PV* dg dp

		*GRANT ELEMENT
		gen ge_calcnew = 0 // This is "grant element calculated", ie. the grant element worked out using the info provided
		replace ge_calcnew = eppnew if typerepayment==1
		replace ge_calcnew = annuitynew if typerepayment==2
		replace ge_calcnew = lumpsumnew if typerepayment==3
		replace ge_calcnew = eppnew if typerepayment==5 //This is an estimate, as need some value for "other" category. Generally not much difference between annuity and EPP, and repayment dates suggest not lump sum
		
		replace ge_calcnew = lumpsumnew if typere==5 & N==. // Investigation showed N only == . when type is lumpsum. Applying similar logic to estimating "other" category (little else can be done)
		replace ge_calcnew = 0 if ge_calcnew<0

}

gen geqnew = ge_calcnew*usd_disbursement

*---------------------------------------------------------
* Re-calculating with threshold (with new discount rates some GEs fall under the 0.45, 0.15, and 0.1 thresholds for LDCs, LMICs, and UMICs)
*I don't use this in blog. It doesn't make much difference because lower discount rates basically mean lots of loans become zero-concessional anyway
*Also, the thresholds were decided partly based on the discount rates so it is a bit unfair to apply after using different discount rates.
*Upshot is that in the estimate presented in blog, some loans have grant element under threshold (again, being generous to DAC)
	gen thresh = 0.45
	replace thresh = 0.15 if group2=="LMICs"
	replace thresh = 0.1 if group2=="UMICs"

	gen ge_calcnew_thresh = ge_calcnew
	replace ge_calcnew_thresh  = 0 if ge_calcnew_thresh < (thresh)  
	gen geqnew_thresh = ge_calcnew_thresh*usd_disbursement
*------------------------------------------------------------

*Create new ODA series. Needs to take into account that private sector loans are cashflow basis, and also some debt forgiveness coded as flowcode==13
gen newoda = geqnew
replace newoda = usd_grantequiv if newoda==.
replace newoda = usd_grantequiv if channelcode>=60000			// this should be same as usd_disbursement
replace newoda = usd_grantequiv if flowcode==13 & finance_t>500 // this should be same as usd_disbursement

*Create new ODA series. Needs to take into account that private sector loans are cashflow basis, and also some debt forgiveness coded as flowcode==13
gen newoda_th = geqnew_thresh
replace newoda_th = usd_grantequiv if newoda_th==.
replace newoda_th = usd_grantequiv if channelcode>=60000			// this should be same as usd_disbursement
replace newoda_th = usd_grantequiv if flowcode==13 & finance_t>500 // this should be same as usd_disbursement

*For my estimate with DAC discount rates, also need to make these adjustments
replace geq = usd_grantequiv if channelcode>=60000
replace geq = usd_grantequiv if flowcode==13 & finance_t>500 

table year if flowcode==13, c(sum usd_grantequiv sum geq sum newoda sum newoda_th) f(%13.0fc)



